.PHONY: clean cleancache cleanall opack
.DEFAULT_GOAL := opack 

# Enable debugging mode if OPACK_DEBUG is set to 'yes'
OPACK_DEBUG=yes

ifndef OPACK_DEBUG
.SILENT:
endif

OPACK_DIR := $(dir $(abspath $(lastword $(MAKEFILE_LIST))))

define OPACK_BANNER 
   ██████  ██ ▄█▀▒███████▒      ▒█████   ██▓███   ██▄       ▄████▄   ██ ▄█▀
 ▒██    ▒  ██▄█▒ ▒ ▒ ▒ ▄▀░     ▒██▒  ██▒▓██░  ██ ▒████▄    ▒██▀ ▀█   ██▄█▒
 ░ ▓██▄   ▓███▄░ ░ ▒ ▄▀▒   ▓▒█ ▒██░  ██▒▓██░ ██▓▒▒██  ▀█▄  ▒▓█    ▄ ▓███▄░
   ▒   ██▒▓██ █▄   ▄▀▒      ░▒ ▒██   ██░▒██▄█▓▒ ▒░██▄▄▄▄██ ▒▓▓▄ ▄██▒▓██ █▄
 ▒██████▒▒▒██▒ █▄▒███████▒░      ████▓▒░▒██▒ ░  ░ ▓█   ▓██▒▒ ▓███▀ ░▒██▒ █▄
 ▒ ▒▓▒ ▒ ░▒ ▒▒ ▓▒░▒▒ ▓░▒░▒░ ░    ▒░▒░▒░ ▒▓▒░ ░  ░ ▒▒   ▓▒█░░ ░▒ ▒  ░▒ ▒▒ ▓▒
 ░ ░▒  ░ ░░ ░▒ ▒░░░ https://git.sk4.nz/sk4nz/skz-opack ▒▒ ░  ░  ▒   ░ ░▒ ▒░
       ░  ░  ░     ░ ░             ░ ░                 ░  ░░ ░      ░  ░
                 ░                                         ░
endef

define spinner
echo -n "$(2) "
($(1)) & pid=$$!; i=0; while ps -a | awk '{print $$1}' | grep -q "$${pid}"; do c=`expr $${i} % 4`; case $${c} in 0) echo -e "◐\c" ;; 1) echo -e "◓\c" ;; 2) echo -e "◑\c" ;; 3) echo -e "◒\c" ;; esac; i=`expr $${i} + 1`; sleep 0.2; echo -e "\b\c"; done; echo; wait $${pid}; ret=$$?; exit $${ret};
endef

$(info $(OPACK_BANNER))

include $(OPACK_DIR)/options.mk
include $(OPACK_DIR)/defines.mk
include $(OPACK_DIR)/packerfile.mk
include $(OPACK_DIR)/packerfile-gcp.mk
include $(OPACK_DIR)/vagrantfile.mk


$(OPACK_PACKER_HTTP_DIR) $(OPACK_PACKER_DIR) ../$(OPACK_CACHE_DIR): 
	mkdir -p $@

$(OPACK_PACKER_DIR)/vagrant.key: | $(OPACK_PACKER_DIR)
	curl -s -o $@ https://raw.githubusercontent.com/hashicorp/vagrant/master/keys/vagrant

$(OPACK_PACKER_DIR)/opack.json: | $(OPACK_PACKER_DIR)
	printf '$(subst $(newline),\n,$(OPACK_PACKER_CONTENT))' > $@

$(OPACK_PACKER_DIR)/bucket.json:
	printf '{ "name": "$(GCE_BUCKET)", "location": "$(GCE_BUCKET_LOCATION)", "storageClass": "STANDARD", "iamConfiguration": {"uniformBucketLevelAccess": { "enabled": true }, } }' > $@

$(OPACK_PACKER_DIR)/opack-cloud.json: | $(OPACK_PACKER_DIR) $(OPACK_PACKER_DIR)/bucket.json
	printf '$(subst $(newline),\n,$(OPACK_PACKER_CLOUD_CONTENT))' > $@
	curl -X POST -s -o /dev/null \
		--data-binary @$(OPACK_PACKER_DIR)/bucket.json \
		-H "Authorization: Bearer $(STORAGE_TOKEN)" \
		-H "Content-Type: application/json" \
		"https://storage.googleapis.com/storage/v1/b?project=$(GCE_PROJECT)"
	curl -X DELETE -s -o /dev/null \
		-H "Authorization: Bearer $(IMAGE_TOKEN)" \
		"https://compute.googleapis.com/compute/v1/projects/$(GCE_PROJECT)/global/images/$(OPACK_TARGET)"

$(OPACK_PACKER_HTTP_DIR)/install.conf: | $(OPACK_PACKER_HTTP_DIR)
	printf '$(subst $(newline),\n,$(OPACK_INSTALL_CONTENT))' > $@

$(OPACK_PACKER_HTTP_DIR)/autodisklabel: | $(OPACK_PACKER_HTTP_DIR)
	cp $(OPACK_AUTODISKLABEL_FILE) $@

$(OPACK_PROVISION_FILE):
	echo you need to write $@ && exit 1

../id_ed25519:
	ssh-keygen -q -t ed25519 -f $@ -N '""'

$(OPACK_BOX_FILE): | $(OPACK_PACKER_DIR)/opack.json $(OPACK_PACKER_DIR)/vagrant.key $(OPACK_PROVISION_FILE) $(OPACK_PACKER_HTTP_DIR)/install.conf $(OPACK_PACKER_HTTP_DIR)/autodisklabel
ifndef OPACK_DEBUG
	$(call spinner,cd $(OPACK_PACKER_DIR) && (CHECKPOINT_DISABLE=1 PACKER_CACHE_DIR=../../$(OPACK_CACHE_DIR) packer build -timestamp-ui opack.json > opack.log && rm -rf $(OPACK_PACKER_DIR)) || echo Error Autoinstalling: check $(OPACK_PACKER_DIR)/opack.log,█ Autoinstalling $(OPACK_BOX_TAG) with Packer)
else
	cd $(OPACK_PACKER_DIR) && CHECKPOINT_DISABLE=1 PACKER_CACHE_DIR=../../$(OPACK_CACHE_DIR) packer build -timestamp-ui opack.json && rm -rf $(OPACK_PACKER_DIR)
endif

$(OPACK_META_FILE): $(OPACK_BOX_FILE)
	printf '$(subst $(OPACK_BOX_FILE_SHA256),$(shell sha256sum $(OPACK_BOX_FILE) | awk '{print $$1}'),$(subst $(newline),\n,$(OPACK_METADATA_CONTENT)))' > $@
ifndef OPACK_DEBUG
	$(call spinner,vagrant box add -f --name $(OPACK_BOX_TAG) $(OPACK_META_FILE) > /dev/null,█ Adding $(OPACK_BOX_FILE) to Vagrant)
else
	vagrant box add -f --name $(OPACK_BOX_TAG) $(OPACK_META_FILE)
endif

$(OPACK_VAGRANT_FILE):
	printf '$(subst $(newline),\n,$(OPACK_VAGRANT_CONTENT))' > $@

opack: $(OPACK_META_FILE) $(OPACK_VAGRANT_FILE)
ifndef OPACK_DEBUG
	$(call spinner,vagrant up > /dev/null,█ Starting $(OPACK_HOSTNAME)-$(OPACK_TARGET) with Vagrant)
else
	vagrant up
endif
	echo "█ skz-opack complete, run 'vagrant ssh' to connect to the machine"

opack-cloud: | $(OPACK_PACKER_DIR)/opack-cloud.json ../id_ed25519 $(OPACK_PROVISION_FILE) $(OPACK_PACKER_HTTP_DIR)/install.conf $(OPACK_PACKER_HTTP_DIR)/autodisklabel
	cd $(OPACK_PACKER_DIR) &&\
		CHECKPOINT_DISABLE=1 PACKER_CACHE_DIR=../../$(OPACK_CACHE_DIR)\
		packer build\
		-timestamp-ui opack-cloud.json | tee -a ../$@ &&\
		rm -rf $(OPACK_PACKER_DIR)

opack-clean: 
ifndef OPACK_DEBUG
	-$(call spinner,vagrant destroy -f >/dev/null; cd terraform_??????? 2>/dev/null && terraform destroy -auto-approve && cd .. && rm -rf terraform_???????; rm -rf $(OPACK_VAGRANT_FILE) opack_build_$(OPACK_SHORT_REV) *.log ssh-config .vagrant *.json,█ Cleaning up)
else
	-vagrant destroy -f
	-cd terraform_??????? 2>/dev/null && terraform destroy -auto-approve && cd .. && rm -rf terraform_???????
	-rm -rf $(OPACK_VAGRANT_FILE) opack_installer_$(OPACK_SHORT_REV) *.log ssh-config .vagrant *.json
endif

opack-cleancache:
ifndef OPACK_DEBUG
	-$(call spinner,vagrant box remove -f --all $(OPACK_BOX_TAG) 2>/dev/null; rm -rf $(OPACK_CACHE_DIR),█ Cleaning up cache)
else
	-vagrant box remove -f --all $(OPACK_BOX_TAG) 2>/dev/null
	-rm -rf $(OPACK_CACHE_DIR)
endif

opack-cleanall: opack-clean opack-cleancache
